#pragma once

#include <string>
#include <vector>
typedef std::vector<std::wstring> stringarray;

/*

SteamerPackage - CAPE-OPEN steam/water property package
Copyright (C) 2010 Jasper van Baten

You can use this source code provided that
 - its use is for academic teaching or academic research purposes, and
 - its use is non-commercial, and
 - you will not distribute COM objects that use any of the CLSIDs in Steamer.idl
 - you properly acknowledge the author

For any other use, please contact jasper@amsterchem.com

For more information, see LICENSE.txt

*/



/*

Jasper, AmsterCHEM, 2010

This class is for Variant data manipilation. The main data type is a VARIANT.

The VARIANT is cleaned up at the destruction of this class, unless it is 
detected that the VARIANT is owned by somebody else. This is when the class
is created from an input variable, or when a return value is requested

For performance, the class is mostly inline.

Please take great care of knowing who owns the content of a Variant and therefore
who is responsible for freeing the content

*/

//ERROR utilities:
std::wstring CO_Error(IDispatch *capeObject,HRESULT hr);
std::wstring HResError(HRESULT hr);

//BSTR vs wide string comparison helper

__inline BOOL SameString(const OLECHAR *str1,const OLECHAR *str2)
 {BOOL res=FALSE;
  if (str1)
   {if (str2) res=(lstrcmpiW(str1,str2)==0);
    else if (*str1==0) res=TRUE;
   }
  else 
   if (!str2) res=TRUE;
   else if (*str2==0) res=TRUE;
  return res;
 }
__inline int StringLen(const OLECHAR *str) {return (str)?lstrlenW(str):0;}

//definition of missing numbers.
// we set outgoing missing numbers to CapeDoubleMissingValue
// incoming missing numbers can be checked using isMissing

#include <float.h> 
//the definition as in a depricated document, but might still be used:
#define OldCapeDoubleEmpty -DBL_MAX
//the new definition is IEEE NaN
extern double CapeDoubleMissingValue;
//this we will use to check for missing values:
#define isMissing(d) ((d==OldCapeDoubleEmpty)||(_isnan(d)))
//this should be called as soon as the module is loaded:
void InitializeCapeDoubleMissingValue();

//CVariant class: wrapper class for VARIANT values

class CVariant
{

private:

   VARIANT value;
   OLECHAR logBuf[32];
   BOOL mustDelete;
   //for array variables:
   LONG lBound,count;
   //keep track of element type during debugging
   #ifdef _DEBUG
   VARTYPE elementType;
   #endif

public:

	//default constructor (result will be destroyed)
    CVariant()
	 {VariantInit(&value);	 
      mustDelete=TRUE;
      count=0;
      #ifdef _DEBUG
      elementType=VT_EMPTY;
      #endif
	 }

	//when an existing VARIANT needs to be analysed, use this constructor
	// you must indicate whether or not the result must be destroyed
	CVariant(VARIANT value,BOOL DestroyWhenDone)
	 {mustDelete=DestroyWhenDone;
	  this->value=value;
	  count=0;
      #ifdef _DEBUG
      elementType=VT_EMPTY; //not checked
      #endif
	 }
	
	~CVariant()
	 {//destructor
	  if (mustDelete) VariantClear(&value);
	 }

    //create a variant array; arrays that are created always index from 0
    void MakeArray(int count,VARTYPE vt)
     {SAFEARRAYBOUND ba;
      //clean up old content
      if (mustDelete) 
       if (value.vt!=VT_EMPTY) 
        VariantClear(&value);
      //set new content
      mustDelete=TRUE;
      ba.cElements=count;
      ba.lLbound=0;
      value.parray=SafeArrayCreate(vt,1,&ba);
      value.vt=VT_ARRAY|vt;
      //keep track of content
      lBound=0;
      this->count=count;
      #ifdef _DEBUG
      elementType=vt;
      #endif
     }

	//when a VARIANT return value is requested, use this function.
	// the result will then not be destroyed
	VARIANT ReturnValue() 
	 {mustDelete=FALSE;
	  return value;
	 }

	//number of elements in an array:    
	int GetCount() 
	 {ATLASSERT(count>=0);
	  return count;
	 }

	//functions to reset the content to something else (re-use a CVariant)
	
	void Set(VARIANT value,BOOL DestroyWhenDone)
	 {//clean up old content
      if (mustDelete) 
       if (this->value.vt!=VT_EMPTY) 
        VariantClear(&this->value);
	  //set to existing VARIANT
	  mustDelete=DestroyWhenDone;
	  this->value=value;
	  count=0;
      #ifdef _DEBUG
      elementType=VT_EMPTY; //not checked
      #endif
	 }
	 
	void Clear(BOOL DestroyWhenDone=TRUE)
	 {//clean up old content
      if (mustDelete) 
       if (value.vt!=VT_EMPTY) 
        VariantClear(&value);
	  value.vt=VT_EMPTY;
	  count=0;
	  mustDelete=DestroyWhenDone;
	 }
	 
	//accessing and setting array elements
	
	void SetDoubleAt(LONG index,double d)
	 {//make sure to have called MakeArray before this
	  ATLASSERT((count>0)&&(index<count));
	  ATLASSERT(lBound==0); //saves an operation; all MakeArray values have lBound=0
	  ATLASSERT(elementType==VT_R8);
	  SafeArrayPutElement(value.parray,&index,&d);
	 }
	 
	double GetDoubleAt(LONG index)
	 {//make sure CheckArray was called before this
	  double d;
	  ATLASSERT((count>0)&&(index<count));
	  ATLASSERT(elementType==VT_R8);
	  index+=lBound;
	  SafeArrayGetElement(value.parray,&index,&d);
	  return d;
	 }

	void SetStringAt(LONG index,BSTR b)
	 {//make sure to have called MakeArray before this
	  ATLASSERT((count>0)&&(index<count));
	  ATLASSERT(lBound==0); //saves an operation; all MakeArray values have lBound=0
	  ATLASSERT(elementType==VT_BSTR);
	  SafeArrayPutElement(value.parray,&index,b);
	 }
	 
	BSTR GetStringAt(LONG index)
	 {//make sure CheckArray was called before this
	  // MAKE SURE TO SysFreeString THE RESULTING STRING
	  BSTR b;
	  ATLASSERT((count>0)&&(index<count));
	  ATLASSERT(elementType==VT_BSTR);
	  index+=lBound;
	  SafeArrayGetElement(value.parray,&index,&b);
	  return b;
	 }

	void SetVariantAt(LONG index,VARIANT v)
	 {//make sure to have called MakeArray before this
	  ATLASSERT((count>0)&&(index<count));
	  ATLASSERT(lBound==0); //saves an operation; all MakeArray values have lBound=0
	  ATLASSERT(elementType==VT_VARIANT);
	  SafeArrayPutElement(value.parray,&index,&v);
	 }

	VARIANT GetVariantAt(LONG index)
	 {//make sure CheckArray was called before this
	  // MAKE SURE TO VariantClear THE RESULTING VARIANT
	  // (the call to VariantInit causes some overhead over directly
	  //  re-using a VARIANT in order to keep the application 
	  //  clear and maintainable. VARIANT arrays are assumed not the
	  //  most time consuming part of the application; most VARIANT
	  //  arrays are outgoing; the incoming ones are only for logging)
	  VARIANT v;
	  VariantInit(&v);
	  ATLASSERT((count>0)&&(index<count));
	  ATLASSERT(elementType==VT_VARIANT);
	  index+=lBound;
	  SafeArrayGetElement(value.parray,&index,&v);
	  return v;
	 }

	void SetLongAt(LONG index,LONG i)
	 {//make sure to have called MakeArray before this
	  ATLASSERT((count>0)&&(index<count));
	  ATLASSERT(lBound==0); //saves an operation; all MakeArray values have lBound=0
	  ATLASSERT(elementType==VT_I4);
	  SafeArrayPutElement(value.parray,&index,&i);
	 }

	LONG GetLongAt(LONG index)
	 {//make sure CheckArray was called before this
	  LONG i;
	  ATLASSERT((count>0)&&(index<count));
	  index+=lBound;
	  ATLASSERT(elementType==VT_I4);
	  SafeArrayGetElement(value.parray,&index,&i);
	  return i;
	 }

	VARIANT Copy()
	 {//return a copy of the current value
	  // (destorying is responsibility of caller)
	  VARIANT v;
	  v.vt=VT_EMPTY;
	  VariantCopy(&v,&value);
	  return v;
	 }

	VARIANT Value()
	 {//this will not destroy the value, use only for values we  
	  // own, such as setting values with SetSinglePhaseProp
	  return value;
	 }
	
	//utility functions
		
	BOOL CheckArray(VARTYPE vt,std::wstring &error)
	 {//sets the low bound and count if check is 
      #ifdef _DEBUG
      elementType=vt;
      #endif
	  // performed ok
	  BOOL res=FALSE;
	  //check type
	  if (value.vt==VT_EMPTY)
	   {//this is ok for an empty array
	    res=TRUE;
	    lBound=count=0;
	   }
	  else if (value.vt!=(vt|VT_ARRAY)) 
	   {//not the type we expected
	    error=L"Expected an array of type ";
	    switch (vt)
	     {case VT_EMPTY: error+=L"EMPTY"; ATLASSERT(0); break;
	      case VT_R8: error+=L"double precision";break;
	      case VT_VARIANT: error+=L"VARIANT";break;
	      case VT_BSTR: error+=L"string";break;
	      case VT_I4: error+=L"long";break;
	      case VT_I2: error+=L"short";break;
	      default: error+=L"<unknown>";ATLASSERT(0);break;
	     }
	   }
	  else
	   {//type is ok, check dimension
	    if (SafeArrayGetDim(value.parray)!=1) error=L"unexpected dimension; 1-dimensional array expected";
	    else
	     {//get lower and upper bound:
	      if (FAILED(SafeArrayGetLBound(value.parray,1,&lBound))) error=L"failed to get lower bound";
	      else if (FAILED(SafeArrayGetUBound(value.parray,1,&count))) error=L"failed to get upper bound";
	      else
	       {//all ok
	        count-=lBound-1;
	        res=TRUE;
	       }
	     }
	   }
	  return res;
	 }

};
